	MODULE engine

	MACRO border n
;	 ld a,n
;	 out (0xfe),a
	ENDM

;	org 0x6000,0

; 60xx - flag
;	b7:changed (updated on reFresh)
;	b6:block update
;	b0..2:layer which from start to draw (top non-mask layer with non-zero tile). 0 mean 'no tiles here'
; 61xx..67xx : data

LayData	equ 0x6000

bgColor	db 0
lColors	ds 8
LayType	db 0	; b0 to b6: bottom layer to top layer: bit set -> this is non-mask layer (base)
TileTab	ds 16

bc2scr	push bc
	ld a,c
	cp 32
	jr c,b2s1
	ld c,0
b2s1	ld a,b
	cp 24
	jr c,b2s2
	xor a
	ld b,a
b2s2	; ld a,b
	rrca
	rrca
	rrca
	and 0xe0
	ld l,a
	ld a,c
	and 0x1f
	or l
	ld l,a
	ld a,b
	and 0x18
	or high LayData
	ld h,a
	pop bc
	ret	

; check layers in box
; checkXY : from BC=YX
; checkHL : from (HL & 0x18FF)=box.adr
checkHL	ld a,h
	and 0x18
	or high LayData
	inc a
	ld h,a
	jr chxyIN
checkXY	call bc2scr
	inc h
chxyIN	ld e,0
	ld d,7
chxyC	ld a,(hl)
	sub 1
	ccf
	rr e
	inc h
	dec d
	jr nz,chxyC
	ld a,h
	sub 7
	ld h,a
	and a
	rr e		; e: b0 to b6 (bottom to top) = 1 when layer is not empty
	ret

; drawSpr: put sprite
; rmSpr: remove sprite (fill with 0)
; BC=YX:topLeft corner, D=layer HL=sprite data (dX,dY,{tile.nums})
rmSpr	ld a,0xaf	; xor a
	jr dsprA
drawSpr	ld a,0x7e	; ld a,(hl)
dsprA	ld (dsprTP),a
	push hl
	call bc2scr
	exx
	pop hl
	ld c,(hl)	; dX
	inc hl
	ld b,(hl)	; dY
	inc hl
dsprCY	exx		; BC=YX, D=lay.nr, HL=lay.adr(0), BC'=dYdX, HL'=spr.data
	ld a,b		; if Y not in scr
	cp 24
	jr nc,dsprSY
	exx
	ld a,c		; A = dX
	exx
	push bc
	push hl
dsprCX	exa		; if X not in scr
	exx
dsprTP	ld a,(hl)
	inc hl
	exx
	ld e,a		; E = (sprite.data)
	ld a,c
	cp 32
	jr nc,dsprSX
	push bc
	push hl
	call setilHL	; set: HL=adr, D=lay, E=tile.num
	pop hl
	pop bc
	inc l
dsprSX	inc c
	exa
	dec a
	jr nz,dsprCX
	pop hl
	pop bc
	ld a,l
	add a,32
	ld l,a
	jr nc,dsprsx1
	ld a,h
	add a,8
	ld h,a
dsprsx1	inc b
	exx
	djnz dsprCY
	ret
dsprSY	exx
	ld a,b
	ld b,0
	add hl,bc
	ld b,a
	exx
	jr dsprsx1

; rmTile : set tile 0 on layer D on coords BC=YX (free cell)
; setTile: set tile E on layer D on coords BC=YX
rmTile	ld e,0
setTile	call bc2scr
setilHL	ld a,d
	and 7
	ret z
	or h
	ld h,a
	ld a,(hl)
	cp e
	ret z
	ld (hl),e
	ld a,h
	and 7
	ld c,a		; c:current level
	ld a,h
	and 0xf8
	ld h,a
	ld a,(hl)
	and 7
	ld b,a
	ld a,c
	cp b
	ccf
	sbc a,a
	and 0x80
	or (hl)
	ld (hl),a
scanBox	ld a,(LayType)	; scan box to find upper non-masked tile
	ld c,a
	ld b,7
	ld a,h
	or 7
	ld h,a
	rl c
scanb1	rl c
	jr nc,scanb2
	ld a,(hl)
	inc a
	jr z,scanb2
	dec a
	jr nz,scanb3
scanb2	dec h
	djnz scanb1
scanb3	ld a,h
	and 7
	ld c,a
	ld a,h
	and 0xf8
	ld h,a
	ld a,(hl)
	and 0xf8
	or c
	ld (hl),a
	ret
	
; in : BC = YX, D = layer
; out : E = tile.nr on this place
getTile	call bc2scr
	ld e,0
	ld a,d
	and 7
	ret z
	or h
	ld h,a
	ld e,(hl)
	ret

; redraw changed boxes
update	ld hl,LayData
rfr_1	bit 7,(hl)
	call nz,outBox
	inc l
	jr nz,rfr_1
	ld a,h
	add a,8
	ld h,a
	cp high LayData + 0x18
	jr c,rfr_1
	border 0
	ret

outBox	border 2
	bit 6,(hl)
	jr nz,obxe1
	IFDEF NoColor
	 ld bc,lColors
	 ld a,(hl)
	 and 7
	 add a,c
	 ld c,a
	 adc a,b
	 sub c
	 ld b,a
	 ld a,(bc)
	 and a
	 jr nz,oboxA
	 ld a,(bgColor)
oboxA	 ld (boxBuf+8),a
	ENDIF
	push hl
	IFDEF UseBuffer
	 ld bc,boxBuf
	ELSE
	 ld a,h
	 and 0x18
	 or 0x40
	 ld b,a
	 ld c,l
	ENDIF
	ld a,(hl)
	and 7
	jr z,obxBack
	or h
	ld h,a
	ld a,(hl)
	sub 1
	call nc,obxSolid
obx1	IFDEF UseBuffer
	 ld bc,boxBuf
	ENDIF
	inc h
	ld a,h
	and 7
	jr z,obxExit
	ld a,(hl)
	sub 1
	call nc,obxMask
	jr obx1
obxBack	xor a		; background box
	ld d,8
	ld e,b
obxb1	ld (bc),a
	IFDEF UseBuffer
	 inc bc
	ELSE
	 inc b
	ENDIF
	dec d
	jr nz,obxb1
	
	IFNDEF NoColor
	IFDEF UseBuffer
	inc bc
	ELSE
	ld a,e
	rra
	rra
	rra
	and 3
	or 0x58
	ld b,a
	ENDIF
	ld a,(bgColor)
	ld (bc),a
	ENDIF
	
	ld b,e
	jr obx1
obxExit	pop hl

	IFDEF UseBuffer	
	ld a,h
	and 0x18
	or 0x40
	ld d,a
	ld e,l
	ld bc,boxBuf
obex1	ld a,(bc)
	ld (de),a
	inc bc
	inc d
	ld a,d
	and 7
	jr nz,obex1
;	IFNDEF NoColor
	ld a,d
	dec a
	rra
	rra
	rra
	and 3
	or 0x58
	ld d,a
	ld a,(bc)
	ld (de),a
;	ENDIF
	ld bc,boxBuf
	ENDIF
	
obxe1	res 7,(hl)
	border 1
	ret

	IFDEF UseBuffer
boxBuf	ds 9
	ENDIF
	
obxSolid
	cp 254		; tile 255 is "invisible, but present"
	ret z
	push hl
	ld e,a
	ld d,0
	ld a,h
	and 7
	ld hl,TileTab
	add a,a
	add a,l
	ld l,a
	adc a,h
	sub l
	ld h,a
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	
	IFNDEF NoColor
	add hl,de
	ENDIF
	
	ex de,hl
	add hl,hl
	add hl,hl
	add hl,hl
	add hl,de	; hl:tile adr [8(9) bytes / tile]
	ld d,8
	ld e,b
obxs1	ld a,(hl)
	ld (bc),a
	inc hl
	IFDEF UseBuffer
	 inc bc
	ELSE
	 inc b
	ENDIF
	dec d
	jr nz,obxs1
	
	IFNDEF NoColor
	IFNDEF UseBuffer
	ld a,e
	rra
	rra
	rra
	and 3
	or 0x58
	ld b,a
	ENDIF
	ld a,(hl)
	ld (bc),a
	ENDIF
	
	ld b,e
	pop hl
	ret

obxMask	cp 254
	ret z
	push hl
	ld e,a
	ld d,0
	ld a,h
	and 7
	ld hl,TileTab
	add a,a
	add a,l
	ld l,a
	adc a,h
	sub l
	ld h,a
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	
	IFNDEF NoColor
	add hl,de
	ENDIF
	
	ex de,hl
	add hl,hl
	add hl,hl
	add hl,hl
	add hl,hl
	add hl,de	; hl:mask tile adr [16(17) bytes / tile]
	ld d,8
	ld e,b
obxm1	ld a,(bc)
	and (hl)
	inc hl
	or (hl)
	inc hl
	ld (bc),a
	IFDEF UseBuffer
	 inc bc
	ELSE
	 inc b
	ENDIF
	dec d
	jr nz,obxm1
	
	IFNDEF NoColor
	ld a,(hl)
	and a
	jr z,obxm2
	IFNDEF UseBuffer
	ld d,a
	ld a,e
	rra
	rra
	rra
	and 3
	or 0x58
	ld b,a
	ld a,d
	ENDIF
	ld (bc),a
	ENDIF
	
obxm2	ld b,e
	pop hl
	ret
	
; set update area
; BC = YX of corner
; DE = dYdX (size)

setArea	ld hl,LayData
seta1	set 6,(hl)
	inc l
	jr nz,seta1
	ld a,h
	add a,8
	ld h,a
	cp high LayData + 0x18
	jr c,seta1
	call bc2scr
seta2	ld b,e
	push hl
seta3	res 6,(hl)
	inc l
	djnz seta3
	pop hl
	ld a,l
	add a,32
	ld l,a
	jr nc,seta4
	ld a,h
	add a,8
	ld h,a
seta4	dec d
	jr nz,seta2
	ret
	
cls	ld hl,LayData
cls1	ld a,(hl)
	and 0x40
	or 0x80
	ld (hl),a
	inc l
	jr nz,cls1
	inc h
	ld bc,0x0700
cls2	ld (hl),c
	inc l
	jr nz,cls2
	inc h
	djnz cls2
	ld a,h
	cp high LayData + 0x18
	jr c,cls1
	ret
	
updAll	ld hl,LayData
	ld b,3
updal1	set 7,(hl)
	inc l
	jr nz,updal1
	ld a,h
	add a,8
	ld h,a
	djnz updal1
	ret
	
reset	ld hl,LayData
	ld b,3
res1	res 7,(hl)
	inc l
	jr nz,res1
	ld a,h
	add a,8
	ld h,a
	djnz res1
	ret

	ENDMODULE