	DEVICE ZXSPECTRUM128


varsTab	equ 0x7800		; vars (256)
bitsTab	equ 0x7900		; bits (32)
mapSpr	equ 0x7920		; big sprite (map) size
mapBuf	equ 0x7922		; buf for unpacked layer (0x300 max)

	org 0x7D7D
	
ngBegin	jp intupt		; 7d7d : int
begin	jp start		; 7d80 : run game
	jp func			; 7d83 : call engine function
	dw gfxAdr		; 7d86 : gfx adr
	
intupt	di
	push af,bc,de,hl
	exx
	exa
	push af,bc,de,hl,ix,iy
	
	ld a,(curBank)
	push af
	
	ld a,0x7f
	in a,(0xfe)
	and 4
	ld a,0xa7
	jr nz,mkey01
mskFlag	and a
	jr c,int01
	call muzGSwitch
	ld a,0x37
mkey01	ld (mskFlag),a

int01	call playMuz
	
	pop af
	call setBank
	
	pop iy,ix,hl,de,bc,af
	exx
	exa
	pop hl,de,bc,af
	ei
	ret
	
; functions (in: A=func.nr, other=params)

func	cp fMax
	ret nc
	push hl
	ld hl,fAdrTab
	and 127
	add a,a
	ADDA h,l
	LDHL
	ex (sp),hl
	ret
	
fAdrTab	dw DEHRUST	; 0 : call DEHRUST (HL,DE)
	dw script.exec	; 1 : exec script (HL)
fMax	equ ($-fAdrTab)/2
	
; banks

m0Bank	ld a,mapBank0
	jr setBank
m1Bank	ld a,mapBank1
	jr setBank
mzBank	ld a,muzBank
	jr setBank
xBank	ld a,scrBank1
	jr setBank
wBank	ld a,scrBank0
	jr setBank
gBank	ld a,gfxBank
setBank	push bc
	and 7
	ld (curBank),a
	or 0x10
	ld bc,0x7ffd
	out (c),a
	pop bc
	ret
	
; in : A = page to set. RET will be through bstRet, with restoring previous page
bankSR	ex (sp),hl		; SP:*,HL
	ld (bsrJmp+1),hl
	ld hl,(curBank-1)	; H = curBank
	ex (sp),hl		; SP:*,curbank
	push hl			; SP:*,curbank,HL
	call setBank
	ld hl,bsrRet
	ex (sp),hl		; SP:*,curbank,retadr
bsrJmp	jp 0
bsrRet	ex (sp),hl		; SP:*,HL ; H = curbank
	push af			; SP:*,HL,AF
	ld a,h
	call setBank
	pop af
	pop hl
	ret

curBank	db 0

callNZ	ld a,h
	or l
	ret z
callHL	jp (hl)

reFresh	
L7Flag	scf
	ret c
	ld a,gfxBank
	call bankSR
	jp engine.update

aniSR
updFlag	and a		; tiles was updated
	call c,reFresh
	ld a,0xa7
	ld (updFlag),a
aniFlag	scf
	ret c
	jp aniChar

; =======
	
start	di
	ld hl,0x6000
	ld de,0x6001
	ld bc,0x1d00
	ld (hl),l
	ldir
	ld hl,intTab
	ld de,intTab+1
	ld bc,0x100
	ld a,h
	ld i,a
	ld (hl),high ngBegin
	ldir
	im 2

gameStart
	ld sp,0x6000
	ei
	call wBank
	ld hl,(scriptAdr)
	call script.exec
	jr gameStart
	
wBack	ret

; misc

scanKeys
	ld a,(hl)
	and a
	ret z
	inc hl
	in a,(0xfe)
	and (hl)
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	jr nz,scanKeys
	ex de,hl
	or 2
	ret
	
inKeys	call scanKeys
	ret z
	jp (hl)

waitKey	halt
	call aniSR
	xor a
	in a,(0xfe)
	cpl
	and 31
	jr z,waitKey
	ret

noKey	halt
	call aniSR
	xor a
	in a,(0xfe)
	cpl
	and 31
	jr nz,noKey
	ret

getRnd	push hl
radr	ld hl,0
	inc hl
	ld a,h
	and 0x3f
	ld h,a
	ld (radr+1),hl
	ld a,r
	xor (hl)
	pop hl
	ret

gradCls	ld de,gradz
	ld b,5
gclsz	call gcls0
	ld a,e
	add a,8
	ld e,a
	adc a,d
	sub e
	ld d,a
	djnz gclsz
gcls0	ld hl,0x4000
gcls1	push de
gcls2	ld a,(de)
	and (hl)
	ld (hl),a
	inc l
	jr nz,gcls2
	inc h
	inc de
	ld a,h
	and 7
	jr nz,gcls2
	pop de
	ld a,h
	cp 0x58
	jr c,gcls1
	ret

gradz	db 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x55
	db 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55
	db 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00
	db 0xAA, 0x00, 0x44, 0x00, 0xAA, 0x00, 0x44, 0x00
	db 0x44, 0x00, 0x00, 0x00, 0x44, 0x00, 0x00, 0x00
	db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00

atrCls	ld b,8
acls0	halt
	ld hl,0x5800
acls1	ld c,(hl)
	ld a,c
	and 0x38
	jr z,acls2
	ld a,c
	sub 8
	ld c,a
acls2	ld a,c
	and 7
	jr z,acls3
	dec c
acls3	ld (hl),c
	inc hl
	ld a,h
	cp 0x5b
	jr c,acls1
	djnz acls0
	ld hl,0x4000
	ld de,0x4001
	ld bc,0x17ff
	ld (hl),l
	ldir
	ret

ldBC	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	ret
	
ldDE	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	ret

outSpr	push bc
	push de
	ld b,0
	ldir
	pop de
	pop bc
	inc d
	ld a,d
	and 7
	jr nz,outSpr
	ld a,e
	add a,32
	ld e,a
	ccf
	sbc a,a
	and 0xf8
	add a,d
	ld d,a
	djnz outSpr
	ret

; jumps
	
jumpTab	add a,a
	ADDA h,l
	LDHL
	jp (hl)

jumpCDE	ld a,(de)
	cp 0xff
	ret z
	inc de
	cp c
	jr z,jcdeY
	inc de
	inc de
	jr jumpCDE
jcdeY	ex de,hl
	LDHL
	ex de,hl
	push de
	ret

; in: A = bit num
; out: HL = bit adr, C = bit mask, A = bit state (0,1), Z/NZ = bit state
bitAdr	ld hl,bitsTab
badr0	ld c,a
	rra
	rra
	rra
	and 0x3f
	ADDA h,l
	ld a,c
	and 7
	ld c,0x01
	jr z,badr2
badr1	rlc c
	dec a
	jr nz,badr1
badr2	ld a,(hl)
	and c
	ret z
	inc a
	ret

; in: A = num
; out: DE = var.adr, A = var.value
getVar	ld de,varsTab
	ADDA d,e
	ld a,(de)
	ret

findZero
	ld a,(hl)
	inc hl
	and a
	jr nz,findZero
	ret

; put number A as text in 3-byte buffer @ hl
putNum2	exa
	scf
	exa
	jr pnum01
putNum	exa
	scf
	exa
	ld b,100
	call putDig
pnum01	ld b,10
	call putDig
	add a,"0"
	ld (hl),a
	inc hl
	ret
putDig	exa
	jr nc,pdigD
	exa
	cp b
	jr c,pdig0
	exa
	and a
pdigD	exa
	ld c,"0"-1
pdig1	inc c
	sub b
	jr nc,pdig1
	add a,b
	ld (hl),c
	inc hl
	ret
pdig0	ld (hl)," "
	inc hl
	ret

; DE = D * E
multDE	ld h,0
	ld l,e
	ld a,d
	ld de,0
mdeL	and a
	ret z
	rra
	jr nc,mdeN
	ex de,hl
	add hl,de
	ex de,hl
mdeN	add hl,hl
	jr mdeL
	
; checkpoint

saveState
	call findHero
	ret nc
	push ix
	pop hl
	ld de,hDSCsav
sst_01	ld bc,chaDSCSize
	ldir
	ret

restState
	call findHero
	ret nc
	push ix
	pop de
	ld hl,hDSCsav
	jr sst_01
	
; ERROR

L7Error	push af
	ld a,2
	out (0xfe),a
	pop af
	di
	halt
	jr $

backScript			; back (..) for magic & potion menu
	tBattleCancel 1
	tEnd
	
	include "L7engine/engine.asm"
	include "L7engine/script.asm"
	include "L7engine/sound.asm"
	include "L7engine/init.asm"
	include "L7engine/map.asm"
	include "L7engine/zones.asm"
	include "L7engine/animator.asm"
	include "L7engine/menu.asm"
	include "L7engine/craft.asm"
	include "L7engine/doors.asm"
	include "L7engine/equipment.asm"
	include "L7engine/npc.asm"
	include "L7engine/items.asm"
	include "L7engine/potion.asm"
	include "L7engine/magic.asm"
	include "L7engine/battle.asm"
	include "L7engine/dehrust_ix.asm"

cpMap	db 0
hDSCsav	ds chaDSCSize

wItem	dw 0xffff
wNPC	dw 0xffff
wElem	dw 0xffff

fbSpace	dw battle.bdatTab	; free space after bdatTab
boxMask	db 0			; bit n = 1 if layer n is impassable

;memMap	db 0,7,4,6,1,3	; gfx, muz, map0, map1, scr0, scr1
layMap	ds 8		; 0,bg,wall,item,npc,hero,top,inf

; HERO DATA

heroData
	dw heroAT	; battle anims tab
eqHero	ds 8
maxHP	db 50		; hp
maxMP	db 20		; mp

hpInit	db 50,20
expa	dw 0
level	db 0

fatkAP	dw 0
fatkIP	dw 0

	align 256
intTab	ds 257
gfxAdr	equ $

; save bin
/*
	PAGE gfxBank
	SAVEBIN "out/main7E7B-0.bin",begin,mainSize
	
	PAGE wrkBank
	SAVEBIN "out/dataC000-1.bin",0xc000,dataSize
*/